# -*- coding: utf-8 -*-

# Authors: Roberto Alsina <roberto.alsina@canonical.com>
#
# Copyright 2011 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Widget to create UDFs in the Windows Install Wizard."""

import ctypes
import os
import sys
import threading
import Queue

from twisted.internet.defer import inlineCallbacks, returnValue
from PyQt4 import QtCore, QtGui
from ubuntuone.controlpanel import backend
from ubuntuone.controlpanel.gui import (
    humanize,
    sign_url,
    FOLDER_INVALID_PATH,
)
from ubuntuone.platform.credentials import CredentialsManagementTool
from ubuntu_sso.qt.gui import SSOWizardPage

from ubuntuone_installer.gui.qt.ui import local_folders_ui
from ubuntuone_installer.gui import (
    LOCAL_FOLDERS_TITLE,
    LOCAL_FOLDERS_SPACE_HEADER,
    LOCAL_FOLDERS_OFFER_LABEL,
    LOCAL_FOLDERS_CALCULATING,
)

from ubuntuone_installer.logger import setup_logging


# Invalid name logger
# pylint: disable=C0103
logger = setup_logging('qt.local_folders')
# pylint: enable=C0103


class FolderItem(QtGui.QTreeWidgetItem):
    """Class representing a folder in the folder list UI."""
    def __init__(self, strings, path=None, queue=None,
        calculate=True, volume_id=None):
        super(FolderItem, self).__init__(strings)
        self.thread = CalculateSize(path, queue)
        # Don't calculate sizes of UDFs
        if calculate and not volume_id:
            self.thread.start()
            self.size = None
        else:
            self.size = 0
        self.path = path
        self.setCheckState(0, QtCore.Qt.Unchecked)
        self.volume_id = volume_id


class CalculateSize(threading.Thread):
    """Class to calculate the size of a folder in the background."""
    def __init__(self, path_name, queue):
        self.path_name = path_name
        self.queue = queue
        self._stop = False
        super(CalculateSize, self).__init__()
        self.daemon = True

    def run(self):
        total_size = 0
        for dirpath, _, filenames in os.walk(self.path_name):
            for f in filenames:
                fp = os.path.join(dirpath, f)
                total_size += os.path.getsize(fp)
        self.queue.put([self.path_name, total_size])


class LocalFoldersPage(SSOWizardPage):
    """Wizard page to create UDFs in the Windows Installer."""

    def __init__(self, parent=None):
        super(LocalFoldersPage, self).__init__(
            local_folders_ui.Ui_Form(), None, parent)
        self.setTitle(LOCAL_FOLDERS_TITLE)
        self.setSubTitle()
        header_view = self.ui.folder_list.header()
        header_view.setResizeMode(0, header_view.Stretch)
        self.queue = Queue.Queue()
        self.timer = QtCore.QTimer()
        self.items = {}
        self.folders_info = None
        self.account_info = None
        self.cp_backend = backend.ControlBackend()

    # initializePage is inherited
    # pylint: disable=C0103
    def initializePage(self):
        """UI details."""
        self.wizard().setButtonLayout([
            QtGui.QWizard.Stretch,
            QtGui.QWizard.NextButton])
        self.wizard()._next_id = self.wizard().SYNC_NOW_OR_LATER_PAGE
        # Start with this invisible
        self.ui.offer_frame.setVisible(False)
        # Show overlay until we have server data
        self.wizard().overlay.show()
        self.get_info()

    @inlineCallbacks
    def get_info(self):
        """Get information from CP backend and fill folder list."""
        # pylint: disable=W0702
        try:
            volumes_info = yield self.cp_backend.volumes_info()
            self.account_info = yield self.cp_backend.account_info()
            self.folders_info = []
            for _, _, volumes in volumes_info:
                for volume in volumes:
                    if volume[u'type'] == u"UDF":
                        self.folders_info.append(volume)
            self.ui.folder_list.clear()
            for folder in self.folders_info:
                item = yield self.add_folder(
                    os.path.expanduser(folder['path']),
                    validate=False,
                    volume_id=folder['volume_id'],
                    calculate=False,
                    )
                if item:
                    if folder['subscribed']:
                        item.setCheckState(0, QtCore.Qt.Checked)
                    item.size = 0
            for folder_name in self.default_folders():
                item = yield self.add_folder(folder_name, validate=True)
            self.timer.start(2000)
            self.timer.timeout.connect(self.update_sizes)
            self.wizard().overlay.hide()
            self.wizard().currentIdChanged.connect(self.changed_page)
        except:
            logger.exception("Error getting backend info:")

    @QtCore.pyqtSlot("int")
    @inlineCallbacks
    def changed_page(self, page_id):
        """When moving to next page, create/[un]subscribe UDFs."""
        self.timer.stop()
        try:
            self.wizard().currentIdChanged.disconnect(self.changed_page)
        except KeyError:
            pass
        if page_id == self.wizard().SYNC_NOW_OR_LATER_PAGE:
            # The page following this one
            self.wizard().overlay.show()
            for path, item in self.items.items():
                if item.checkState(0) == QtCore.Qt.Checked:
                    if item.volume_id:
                        yield self.cp_backend.change_volume_settings(
                            item.volume_id,
                            dict(subscribed=True))
                    else:
                        yield self.cp_backend.create_folder(path)
                else:
                    if item.volume_id:
                        yield self.cp_backend.change_volume_settings(
                            item.volume_id,
                            dict(subscribed=False))
            self.wizard().overlay.hide()

    def default_folders(self):
        """Return a list of the folders to add by default."""
        if sys.platform == 'win32':
            # XXXX to be replaced by calls to xdg_base_directory's
            # special_folders
            dll = ctypes.windll.shell32
            buf = ctypes.create_string_buffer(300)
            dll.SHGetSpecialFolderPathA(None, buf, 5, False)
            docs = buf.value
            dll.SHGetSpecialFolderPathA(None, buf, 13, False)
            music = buf.value
            dll.SHGetSpecialFolderPathA(None, buf, 39, False)
            pictures = buf.value
            return [docs, music, pictures]
        else:
            result = ['To be implemented']
        return result

    @inlineCallbacks
    def add_folder(self, path, validate=True, calculate=True, volume_id=None):
        """Add a folder to the list."""
        if path in self.items:
            returnValue(None)
        if validate:
            is_valid = yield self.cp_backend.validate_path_for_folder(path)
        else:
            is_valid = True
        if is_valid:
            item = FolderItem([path, ""],
                path=path,
                queue=self.queue,
                volume_id=volume_id,
                calculate=calculate)
            self.ui.folder_list.addTopLevelItem(item)
            self.items[path] = item
            returnValue(item)
        returnValue(None)

    def update_sizes(self):
        """Poll the queue were the threads put the size info."""
        while True:
            try:
                path, size = self.queue.get(False)
            except Queue.Empty:
                break
            else:
                item = self.items.get(path)
                if item:
                    item.size = size
                    try:
                        item.setText(1, humanize(size))
                    except RuntimeError:
                        del self.items[path]
        total = long(self.account_info['quota_used'])
        for path, item in self.items.items():
            if item.size is None:
                total = LOCAL_FOLDERS_CALCULATING
                break
            if not item.volume_id and item.checkState(0) == QtCore.Qt.Checked:
                # Existing UDFs are already accounted for, count if marked.
                total += item.size
        if isinstance(total, long):
            self.show_hide_offer(total)
            total = humanize(total)
        else:
            self.show_hide_offer(0)
        self.ui.folder_list.headerItem().setText(
            1, LOCAL_FOLDERS_SPACE_HEADER % total)

    def show_hide_offer(self, cur_size):
        """Show or hide the offer to buy space according to the total size."""
        quota = self.account_info['quota_total']
        if cur_size > quota:
            self.ui.offer_frame.setVisible(True)
        else:
            self.ui.offer_frame.setVisible(False)
        self.ui.offer_label.setText(LOCAL_FOLDERS_OFFER_LABEL %
            {"quota": humanize(quota)})

    def stop_threads(self):
        """Stop all pending threads."""
        for _, item in self.items:
            item.thread._stop = True

    # itemClicked is a Qt signal name.
    # pylint: disable=C0103
    def on_folder_list_itemChanged(self, item, column):
        """Delete folder from the list."""
        if column == 0:
            self.update_sizes()

    @inlineCallbacks
    @QtCore.pyqtSlot()
    def on_add_storage_button_clicked(self):
        """user clicked on the "Add more storage" button."""
        # Really want to catch everything
        # pylint: disable=W0702
        url = "https://one.ubuntu.com/services/#storage_panel"
        try:
            credtool = CredentialsManagementTool()
            creds = yield credtool.find_credentials()
        except:
            logger.exception('Error while trying to get credentials:')
            creds = {}
        if creds:
            signed_url = sign_url(url, creds)
        else:
            signed_url = url
        QtGui.QDesktopServices.openUrl(QtCore.QUrl(signed_url))

    @inlineCallbacks
    @QtCore.pyqtSlot()
    def on_add_folder_button_clicked(self):
        """user clicked on the "Add Folder" button."""
        folder = QtGui.QFileDialog.getExistingDirectory(parent=self)
        folder = unicode(folder)
        if folder == '':
            return

        is_valid = yield self.cp_backend.validate_path_for_folder(folder)
        if not is_valid:
            user_home = os.path.expanduser('~')
            text = FOLDER_INVALID_PATH % {'folder_path': folder,
                                          'home_folder': user_home}
            QtGui.QMessageBox.warning(self, '', text, QtGui.QMessageBox.Close)
            return
        yield self.add_folder(folder, validate=False, volume_id=False)
