# -*- coding: utf-8 -*-

# Authors: Roberto Alsina <roberto.alsina@canonical.com>
#
# Copyright 2011 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Customized Setup Account page for SSO."""

import gettext
import re

from PyQt4 import QtGui, QtCore

from ubuntu_sso.qt import gui as sso_gui

_ = gettext.gettext

# pylint: disable=C0103
BAD = u'<img src=":/password_hint_warning.png" /><font> %s </font>'
ERROR = u'<font color="#dd4814"><b> %s </b></font>'
GOOD = u'<img src=":/password_hint_ok.png" /><font> %s </font>'
NORMAL = u'<font> %s </font>'
TITLE_STYLE = "<span style=\"font-size:24px\">%s</span>"

EMPTY_NAME = _("Please enter your name")
INVALID_EMAIL = _("Please enter a valid email address")
EMAIL_MATCH = _("The email addresses do not match")
PASSWORD_LENGTH = _("At least 8 characters")
PASSWORD_DIGIT = _("At least one number")
PASSWORD_UPPER = _("At least one uppercase letter")
PASSWORD_MATCH = _("Passwords don't match")


class SetupAccountPage(sso_gui.SetupAccountPage):
    """Customized Setup Account page for SSO."""

    def __init__(self, *args, **kwargs):
        super(SetupAccountPage, self).__init__(*args, **kwargs)
        self.ui.name_edit.textEdited.connect(self.name_assistance)
        self.ui.email_edit.textEdited.connect(self.email_assistance)
        self.ui.email_edit.textEdited.connect(self.confirm_email_assistance)
        self.ui.confirm_email_edit.textEdited.connect(
            self.confirm_email_assistance)
        self.ui.password_edit.textEdited.connect(
            lambda: self.password_assistance(NORMAL))
        self.ui.confirm_password_edit.textEdited.connect(
            self.password_assistance)

    # Invalid name "initializePage"
    # pylint: disable=C0103

    def initializePage(self):
        """Setup UI details."""
        # We need to override some texts from SSO
        # to match our spec
        title_page = TITLE_STYLE % _("Sign Up to Ubuntu One")
        self.setTitle(title_page)
        self.setSubTitle(_("You only need to set up your account "
        "once to get access to Ubuntu One across your devices."))

        self.ui.name_label.setText(_("Name"))
        self.ui.email_label.setText(_("Email"))
        self.ui.confirm_email_label.setText(_("Retype email"))
        self.ui.password_label.setText(_("Create a password"))
        self.ui.confirm_password_label.setText(_("Retype password"))
        self.ui.terms_checkbox.setText(
            _("By signing up to Ubuntu One you agree to our Terms "
            "of Service and Privacy Policy"))
        self.ui.terms_button.setText(_("Show Terms of Service"))
        self.ui.password_info_label.hide()

        # Button setup
        self.wizard().setButtonLayout([
            QtGui.QWizard.BackButton,
            QtGui.QWizard.Stretch])

        self.password_default_assistance()
        # Hide assistance labels by default
        self.ui.name_assistance.setVisible(False)
        self.ui.email_assistance.setVisible(False)
        self.ui.confirm_email_assistance.setVisible(False)
        self.ui.password_assistance.setVisible(False)
        self.ui.refresh_label.setVisible(True)

    def focus_changed(self, old, now):
        """Check who has the focus to activate password popups if necessary."""
        if now == self.ui.password_edit:
            self.ui.password_assistance.setVisible(True)
        elif old == self.ui.password_edit or \
             now == self.ui.confirm_password_edit:
            self.ui.password_assistance.setVisible(True)
            self.password_assistance()

    def name_assistance(self):
        """Show help for the name field."""
        text = unicode(self.ui.name_edit.text())
        if not text.strip():
            self.ui.name_assistance.setVisible(True)
            self.ui.name_assistance.setText(ERROR % EMPTY_NAME)
            self.check_as_invalid(self.ui.name_edit)
        else:
            self.ui.name_assistance.setVisible(False)
            self.check_as_valid(self.ui.name_edit)

    def email_assistance(self):
        """Show help for the email field."""
        text = unicode(self.ui.email_edit.text())
        if not is_correct_email(text):
            self.ui.email_assistance.setText(ERROR % INVALID_EMAIL)
            self.ui.email_assistance.setVisible(True)
            self.check_as_invalid(self.ui.email_edit)
        else:
            self.ui.email_assistance.setVisible(False)
            self.check_as_valid(self.ui.email_edit)

    def confirm_email_assistance(self):
        """Show help for the confirm email field."""
        text1 = unicode(self.ui.email_edit.text())
        text2 = unicode(self.ui.confirm_email_edit.text())
        if text1 != text2:
            self.ui.confirm_email_assistance.setText(ERROR % EMAIL_MATCH)
            self.ui.confirm_email_assistance.setVisible(True)
            self.check_as_invalid(self.ui.confirm_email_edit)
        else:
            self.ui.confirm_email_assistance.setVisible(False)
            self.check_as_valid(self.ui.confirm_email_edit)

    def password_assistance(self, icon_type=BAD):
        """Show help for the password field."""
        text1 = unicode(self.ui.password_edit.text())
        text2 = unicode(self.ui.confirm_password_edit.text())
        label_text = ["<b>%s</b>" % _("Your password must contain"), ]

        if len(text1) < 8:
            sign = icon_type
        else:
            sign = GOOD
        label_text.append(sign % PASSWORD_LENGTH)

        if re.search('[A-Z]', text1) is None:
            sign = icon_type
        else:
            sign = GOOD
        label_text.append(sign % PASSWORD_UPPER)

        if re.search('[\d+]', text1) is None:
            sign = icon_type
        else:
            sign = GOOD
        label_text.append(sign % PASSWORD_DIGIT)

        if text1 != text2:
            label_text.append(icon_type % PASSWORD_MATCH)

        self.ui.password_assistance.setText("<br>".join(label_text))

    def password_default_assistance(self):
        """Show default help for the password field."""
        label_text = ["<b>%s</b>" % _("Your password must contain"), ]

        label_text.append(NORMAL % PASSWORD_LENGTH)
        label_text.append(NORMAL % PASSWORD_UPPER)
        label_text.append(NORMAL % PASSWORD_DIGIT)

        self.ui.password_assistance.setText("<br>".join(label_text))

    def check_as_invalid(self, line_edit):
        """Set QLineEdit's formError property as True, refresh the style."""
        line_edit.setProperty("formError", True)
        line_edit.style().unpolish(line_edit)
        line_edit.style().polish(line_edit)

    def check_as_valid(self, line_edit):
        """Set QLineEdit's formError property as False, refresh the style."""
        line_edit.setProperty("formError", False)
        line_edit.style().unpolish(line_edit)
        line_edit.style().polish(line_edit)

    def showEvent(self, event):
        """Set set_up_button as default button when the page is shown."""
        # This method should stays here because if we move it to initializePage
        # set_up_button won't take the proper style for hover and press
        self.ui.set_up_button.setDefault(True)
        self.ui.set_up_button.setProperty("DisabledState", True)
        self.ui.set_up_button.style().unpolish(self.ui.set_up_button)
        self.ui.set_up_button.style().polish(self.ui.set_up_button)
        self.connect(QtGui.QApplication.instance(),
            QtCore.SIGNAL("focusChanged(QWidget*, QWidget*)"),
            self.focus_changed)

    def hideEvent(self, event):
        """Disconnect the focusChanged signal when the page change."""
        try:
            self.disconnect(QtGui.QApplication.instance(),
                QtCore.SIGNAL("focusChanged(QWidget*, QWidget*)"),
                self.focus_changed)
        except TypeError:
            pass


def is_min_required_password(password):
    """Return if the password meets the minimum requirements."""
    if (len(password) < 8 or
        re.search('[A-Z]', password) is None or
        re.search('\d+', password) is None):
        return False
    return True


# FIXME: this should do the same check as SSO's server side
def is_correct_email(email_address):
    """Return if the email is correct."""
    return '@' in email_address
