# -*- coding: utf-8 -*-

# Authors: Alejandro J. Cura <alecu@canonical.com>
#          Roberto Alsina <roberto.alsina@canonical.com>
#
# Copyright 2011 Canonical Ltd.
#
# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

"""Tests for the Qt UI."""

import os

from twisted.internet import defer
from PyQt4 import QtGui, QtCore
from ubuntuone.devtools.testcase import skipIfOS
from ubuntuone.platform.credentials import APP_NAME
from ubuntuone.controlpanel.gui import qt

from ubuntuone_installer.gui.qt import gui
from ubuntuone_installer.gui.qt.tests import BaseTestCase
from ubuntuone_installer.gui.qt import forgotten
from ubuntuone_installer.gui.qt import setup_account
from ubuntuone_installer.gui.qt.ui import (
    setup_account_ui,
)
from ubuntuone_installer.gui.qt import utils


TOKEN = {u'consumer_key': u'xQ7xDAz',
         u'consumer_secret': u'KzCJWCTNbbntwfyCKKjomJDzlgqxLy',
         u'token_name': u'test',
         u'token': u'GkInOfSMGwTXAUoVQwLUoPxElEEUdhsLVNTPhxHJDUIeHCPNEo',
         u'token_secret': u'qFYImEtlczPbsCnYyuwLoPDlPEnvNcIktZphPQklAWrvyfFMV'}


class FakeFailureCredentialsManagementTool(object):

    """A fake CredentialsManagementTool that fails."""

    def find_credentials(self):
        """Return a deferred that never triggers callback."""
        return defer.succeed({})


class FakeController(object):

    """A fake controller for the tests."""

    def __init__(self, *args, **kwargs):
        self.args = (args, kwargs)

    # pylint: disable=C0103
    def setupUi(self, view):
        """Fake the setup."""
    # pylint: enable=C0103
    is_fake = True


class FakeAreYouSure(object):

    """A fake Dialog."""

    result = 0
    shown = False

    def __init__(self, *args, **kwargs):
        self.args = (args, kwargs)

    def exec_(self):
        """Fake showing the dialog."""
        self.shown = True
        return self.result  # Means reject


class FakeOverlay(object):

    """A fake delay overlay."""

    def __init__(self, *args, **kwargs):
        """Initialize."""
        self.show_counter = 0
        self.hide_counter = 0
        self.args = (args, kwargs)

    def show(self):
        """Fake show."""
        self.show_counter += 1

    def hide(self):
        """Fake hide."""
        self.hide_counter += 1

    def resize(self, *args):
        """Fake resize."""


class FakeLocalFoldersPage(QtGui.QWizardPage):
    """Fake Local Folders Page."""


class FakeUIFoldersPanel(QtGui.QWidget):
    """Fake UI Folders Page."""

    def __init__(self, parent=None):
        QtGui.QWidget.__init__(self, parent)
        self.share_publish_button = QtGui.QPushButton()
        self.add_folder_button = QtGui.QPushButton()
        self.folders = QtGui.QTreeWidget()
        self.folders.setColumnCount(3)
        self.folders.setColumnHidden(2, True)


class FakeFoldersPanel(QtGui.QWidget):
    """Fake Folders Page."""

    ui = FakeUIFoldersPanel()


class FakeUIPreferencesPanel(QtGui.QWidget):
    """Fake UI Preferences Page."""

    apply_changes_button = QtGui.QPushButton()
    restore_defaults_button = QtGui.QPushButton()


class FakePreferencesPanel(QtGui.QWidget):
    """Fake Preferences Page."""

    ui = FakeUIPreferencesPanel()
    apply_button_clicked = False
    restore_button_clicked = False

    def on_apply_changes_button_clicked(self):
        """Set apply_button_clicked to True for PreferencesPage."""
        self.apply_button_clicked = True

    def on_restore_defaults_button_clicked(self):
        """Set restore_button_clicked to True for PreferencesPage."""
        self.restore_button_clicked = True


class MainWindowTestCase(BaseTestCase):
    """Test the qt main window."""

    class_ui = gui.MainWindow

    def setUp(self):
        """Initialize this test instance."""
        # Faking each SSO object instead of doing it lower
        # so we don't rely on any SSO behaviour
        self.patch(gui, "SetUpAccountController", FakeController)
        self.patch(gui, "TosController", FakeController)
        self.patch(gui, "EmailVerificationController", FakeController)
        self.patch(gui, "CurrentUserController", FakeController)
        self.patch(gui, "SuccessController", FakeController)
        self.patch(gui, "ForgottenPasswordController", FakeController)
        self.patch(gui, "ResetPasswordController", FakeController)
        self.patch(gui, "AreYouSure", FakeAreYouSure)
        self.patch(gui, "LoadingOverlay", FakeOverlay)
        self.patch(gui, "LocalFoldersPage", FakeLocalFoldersPage)
        self.patch(qt.preferences, "PreferencesPanel", FakePreferencesPanel)
        self.patch(qt.folders, "FoldersPanel", FakeFoldersPanel)
        self.patch(gui, "CredentialsManagementTool",
            FakeFailureCredentialsManagementTool)
        super(MainWindowTestCase, self).setUp()

    def test_execute_uninstall_on_licence_cancel(self):
        """Pressing Disagree button from license page the uninstall is exec."""
        self.ui.setStartId(self.ui.LICENSE_PAGE_ID)
        self.ui.restart()
        self.ui.show()
        gui.AreYouSure.result = 0
        self.patch(utils, "uninstall_application", self._set_called)
        self.ui.done(result=0)
        self.assertTrue(self._called)

    def test_execute_uninstall_on_licence_cancel_frozen(self):
        """Pressing Disagree button from license page the uninstall is exec."""
        self.ui.setStartId(self.ui.LICENSE_PAGE_ID)
        self.ui.restart()
        self.ui.show()
        gui.AreYouSure.result = 0
        self.patch(utils, "uninstall_application", self._set_called)
        self.ui.done(result=0)
        self.assertTrue(self._called)

    def test_preferences_page(self):
        """Test Preferences Page UI display."""
        preferences_page = self.ui.page(self.ui.preferences_page_id)

        # Show the preferences page
        self.ui.setStartId(self.ui.preferences_page_id)
        self.ui.restart()
        self.ui.show()

        btn_pref = preferences_page.preferences_widget.ui.apply_changes_button
        self.assertEqual(btn_pref.isVisible(), False)
        btn_pref = \
            preferences_page.preferences_widget.ui.restore_defaults_button
        self.assertEqual(btn_pref.isVisible(), False)

    def test_preferences_page_navigation_buttons(self):
        """Check Preferences Page navigation buttons text."""
        preferences_page = self.ui.page(self.ui.preferences_page_id)

        # Show the preferences page
        self.ui.setStartId(self.ui.preferences_page_id)
        self.ui.restart()
        self.ui.show()

        btn_pref = preferences_page.wizard().button(
            QtGui.QWizard.CustomButton1)
        self.assertEqual(btn_pref.text(), "Apply these settings >")
        btn_pref = preferences_page.wizard().button(
            QtGui.QWizard.CustomButton2)
        self.assertEqual(btn_pref.text(), "Default settings")

    def test_preferences_page_button_clicked(self):
        """Check Preferences Page navigation buttons text."""
        preferences_page = self.ui.page(self.ui.preferences_page_id)

        # Show the preferences page
        self.ui.setStartId(self.ui.preferences_page_id)
        self.ui.restart()
        self.ui.show()

        # pylint: disable=W0212
        preferences_page._button_clicked(QtGui.QWizard.CustomButton2)
        self.assertEqual(
            preferences_page.preferences_widget.restore_button_clicked, True)
        preferences_page._button_clicked(QtGui.QWizard.CustomButton1)
        self.assertEqual(
            preferences_page.preferences_widget.apply_button_clicked, True)
        # for Cleanup
        btn_pref = preferences_page.wizard().button(QtGui.QWizard.NextButton)
        self.assertEqual(btn_pref.text(), "Sync Now!")
        self.assertEqual(self.ui._next_id, self.ui.CONGRATULATIONS_PAGE)

    def test_folders_page(self):
        """Check Folders Page UI."""
        folders_page = self.ui.page(self.ui.folders_page_id)

        # Show the preferences page
        self.ui.setStartId(self.ui.folders_page_id)
        self.ui.restart()
        self.ui.show()

        btn_folders = folders_page.folders_widget.ui.share_publish_button
        self.assertEqual(btn_folders.isVisible(), False)
        btn_folders = folders_page.folders_widget.ui.add_folder_button
        self.assertEqual(btn_folders.isVisible(), False)
        tree_folders = folders_page.folders_widget.ui.folders
        self.assertEqual(tree_folders.isHeaderHidden(), True)
        tree_folders = folders_page.folders_widget.ui.folders
        self.assertEqual(tree_folders.isColumnHidden(2), True)
        # pylint: disable=W0212
        self.assertEqual(self.ui._next_id, self.ui.CONGRATULATIONS_PAGE)

    def test_folders_page_next_id(self):
        """Check Folders Page UI."""
        folders_page = self.ui.page(self.ui.folders_page_id)

        # Show the preferences page
        self.ui.setStartId(self.ui.folders_page_id)
        self.ui.restart()
        self.ui.show()

        folders_page.on_check_settings_button_clicked()
        self.assertEqual(self.ui.currentId(), self.ui.preferences_page_id)

    def test_critical(self):
        """Show a critical popup, but first hide the overlay."""
        self.assertEqual(self.ui.overlay.hide_counter, 1)
        self.ui.critical("WORRY!")
        self.assertEqual(self.ui.overlay.hide_counter, 2)
        self.assertEqual(self.ui.form_errors_label.text(), "WORRY!")

    def test_forgotten_password_controller_error(self):
        """When there is an error, it should hide the overlay."""
        self.assertEqual(self.ui.overlay.hide_counter, 1)
        controller = forgotten.ForgottenPasswordController()
        controller.view = self.ui.forgotten
        controller.on_password_reset_error("app_name",
            {'errtype': 'ResetPasswordTokenError'})
        self.assertEqual(self.ui.overlay.hide_counter, 2)

    def test_overlay_connection_setup_account(self):
        """Make sure we don't lose connections to the overlay."""
        self.assertEqual(self.ui.overlay.show_counter, 0)
        # This test is different from the ones below.
        # For some reason, click() doesn't trigger the connected
        # slot, but clicked.emit() does. The reason is unknown.
        self.ui.setup_account.ui.set_up_button.clicked.emit(False)
        self.assertEqual(self.ui.overlay.show_counter, 1)

    def test_overlay_connection_email_verification(self):
        """Make sure we don't lose connections to the overlay."""
        self.assertEqual(self.ui.overlay.show_counter, 0)
        self.ui.email_verification.next_button.click()
        self.assertEqual(self.ui.overlay.show_counter, 1)

    def test_overlay_connection_current_user(self):
        """Make sure we don't lose connections to the overlay."""
        self.assertEqual(self.ui.overlay.show_counter, 0)
        self.ui.current_user.ui.sign_in_button.clicked.emit(False)
        self.assertEqual(self.ui.overlay.show_counter, 1)

    def test_overlay_connection_forgotten_password(self):
        """Make sure we don't lose connections to the overlay."""
        self.assertEqual(self.ui.overlay.show_counter, 0)
        self.ui.forgotten.ui.send_button.clicked.emit(False)
        self.assertEqual(self.ui.overlay.show_counter, 1)

    def test_overlay_connection_reset_password(self):
        """Make sure we don't lose connections to the overlay."""
        self.assertEqual(self.ui.overlay.show_counter, 0)
        self.ui.reset_password.ui.reset_password_button.clicked.emit(False)
        self.assertEqual(self.ui.overlay.show_counter, 1)

    def test_next_hides_overlay(self):
        """Make sure next() hides the overlay."""
        self.ui.setStartId(self.ui.LICENSE_PAGE_ID)
        self.ui.restart()
        self.ui.show()
        self.ui.next()
        # This is 2 because we hide it once on __init__()
        # and then again on next()
        self.assertEqual(self.ui.overlay.hide_counter, 2)

    def test_insert_remove_critical_message(self):
        """Make sure critical message is added and next() removes it."""
        self.ui.setStartId(self.ui.LICENSE_PAGE_ID)
        self.ui.restart()
        self.ui.show()
        license_page = self.ui.page(self.ui.LICENSE_PAGE_ID)
        self.assertEqual(self.ui.form_errors_label.parentWidget(), None)
        self.ui.critical("test critical message")
        self.assertEqual(self.ui.form_errors_label.parentWidget(),
            license_page)
        self.assertEqual(self.ui.form_errors_label.text(),
            "test critical message")
        self.ui.next()
        self.assertEqual(self.ui.form_errors_label.parentWidget(), None)

    def test_done_calls_custom_close_callback(self):
        """When closing the window, close_callback is called."""
        gui.AreYouSure.result = 0
        self.ui.close_callback = self._set_called
        self.ui.done(result=0)
        self.assertEqual(self._called,
                         ((), {}), 'close_callback called.')

    def test_close_callback_can_be_none(self):
        """The close_callback can be None."""
        gui.AreYouSure.result = 0
        self.ui.close_callback = None
        self.ui.done(result=0)
        # world did not explode

    def test_main_window_general_config(self):
        """Check if the main properties of the windows are set."""
        self.assertEqual(self.ui.windowTitle(), APP_NAME)

    def test_start_control_panel_on_finishing(self):
        """Calling done with result=1, the control panel should be called."""
        gui.AreYouSure.result = 0
        self.patch(gui.subprocess, "Popen", self._set_called)
        # If it's frozen this test fails because the path
        # depends on the location of the exe
        if hasattr(gui.sys, "frozen"):
            delattr(gui.sys, "frozen")
        self.ui.done(result=1)
        self.assertEqual(self._called,
            ((['ubuntuone-control-panel-qt'],), {}))

    @skipIfOS('linux2', 'Windows-specific tests.')
    def test_start_control_panel_frozen(self):
        """When frozen, the control-panel has a path."""
        self.ui.setStartId(self.ui.SYNC_NOW_OR_LATER_PAGE)
        self.ui.restart()
        self.ui.show()
        gui.AreYouSure.result = 0
        # I can't patch sys because frozen is not there by default
        gui.sys.frozen = True
        self.patch(gui.subprocess, "Popen", self._set_called)
        self.ui.done(result=1)
        self.assertIn(os.sep, self._called[0][0][0])
        self.assertIn('ubuntuone-control-panel-qt', self._called[0][0][0])

    def test_not_start_control_panel_on_cancel(self):
        """Called done with result=0, control panel should NOT be called."""
        gui.AreYouSure.result = 0
        self.patch(gui.subprocess, "Popen", self._set_called)
        self.ui.done(result=0)
        self.assertEqual(self._called, False)

    def test_continue_cancels_cancel(self):
        """Clicking 'Continue' cancels cancellation."""
        gui.AreYouSure.result = 1
        self.patch(gui.subprocess, "Popen", self._set_called)
        self.ui.show()
        self.ui.done(result=0)
        self.assertEqual(self.ui.isVisible(), True)

    def test_user_is_sure_in_last_page(self):
        """Cancelling on the last page is ok."""
        self.ui.show()
        self.patch(self.ui, 'currentId', lambda: self.ui.CONGRATULATIONS_PAGE)
        self.patch(gui.subprocess, "Popen", self._set_called)
        self.ui.done(result=0)
        self.assertEqual(self._called, False)
        self.assertEqual(FakeAreYouSure.shown, False)

    def test_later_accepts_cancel(self):
        """Clicking 'Later' accepts cancellation."""
        self.ui.setStartId(self.ui.SYNC_NOW_OR_LATER_PAGE)
        self.ui.restart()
        self.ui.show()
        gui.AreYouSure.result = 0
        self.ui.show()
        self.ui.done(result=0)
        self.assertEqual(self.ui.isVisible(), False)

    def test_sync_now_shows_message(self):
        """If the user selects 'Sync Now' the congrats page shows a message."""
        congrats_page = self.ui.page(self.ui.CONGRATULATIONS_PAGE)
        congrats_page.setField("sync_now", True)

        # Show the congrats page
        self.ui.setStartId(self.ui.CONGRATULATIONS_PAGE)
        self.ui.restart()
        self.ui.show()

        # Check the properties have the right values
        self.assertEqual(congrats_page.field("sync_later").toBool(),
            False)
        self.assertEqual(
            congrats_page.ui.progressContainer.isVisible(), True)

    def test_sync_page_custom_field(self):
        """If the user selects 'Sync Custom' we move to Folders page."""
        sync_page = self.ui.page(self.ui.SYNC_NOW_OR_LATER_PAGE)
        sync_page.setField("sync_custom", True)

        # Show the Sync Now page
        self.ui.setStartId(self.ui.SYNC_NOW_OR_LATER_PAGE)
        self.ui.restart()
        self.ui.show()

        # Check the navigation is correct
        self.assertEqual(sync_page.nextId(), self.ui.folders_page_id)

    def test_sync_later_hides_message(self):
        """Selecting 'Sync Later' the congrats page hides a message."""
        congrats_page = self.ui.page(self.ui.CONGRATULATIONS_PAGE)
        congrats_page.setField("sync_later", True)

        # Show the congrats page
        self.ui.setStartId(self.ui.CONGRATULATIONS_PAGE)
        self.ui.restart()
        self.ui.show()

        # Check the properties have the right values
        self.assertEqual(congrats_page.field("sync_now").toBool(),
            False)
        self.assertEqual(
            congrats_page.ui.progressContainer.isVisible(), False)

    def test_current_user_controller_parameters(self):
        """Compare controller parameters with expected values."""
        self.assertEqual(self.ui.current_user_controller.args,
        ((), {'message_box': self.ui,
              'title': gui.TITLE_STYLE % gui.SIGN_IN,
              'subtitle': gui.SIGN_IN_SUBTITLE}))

    def test_stage_progression_1(self):
        """Check that each page in the wizard sets the correct stage."""
        self.ui.on_currentIdChanged(self.ui.LICENSE_PAGE_ID)
        self.assertEqual(self.ui.sideWidget().stage, 0)

    def test_stage_progression_2(self):
        """Check that each page in the wizard sets the correct stage."""
        self.ui.on_currentIdChanged(self.ui.SIGNIN_PAGE)
        self.assertEqual(self.ui.sideWidget().stage, 1)

    def test_stage_progression_3(self):
        """Check that each page in the wizard sets the correct stage."""
        self.ui.on_currentIdChanged(self.ui.setup_account_page_id)
        self.assertEqual(self.ui.sideWidget().stage, 1)

    def test_stage_progression_4(self):
        """Check that each page in the wizard sets the correct stage."""
        self.ui.on_currentIdChanged(self.ui.tos_page_id)
        self.assertEqual(self.ui.sideWidget().stage, 1)

    def test_stage_progression_5(self):
        """Check that each page in the wizard sets the correct stage."""
        self.ui.on_currentIdChanged(self.ui.email_verification_page_id)
        self.assertEqual(self.ui.sideWidget().stage, 1)

    def test_stage_progression_6(self):
        """Check that each page in the wizard sets the correct stage."""
        self.ui.on_currentIdChanged(self.ui.current_user_page_id)
        self.assertEqual(self.ui.sideWidget().stage, 1)

    def test_stage_progression_7(self):
        """Check that each page in the wizard sets the correct stage."""
        self.ui.on_currentIdChanged(self.ui.SUCCESS_PAGE)
        self.assertEqual(self.ui.sideWidget().stage, 1)

    def test_stage_progression_8(self):
        """Check that each page in the wizard sets the correct stage."""
        self.ui.on_currentIdChanged(self.ui.ERROR_PAGE)
        self.assertEqual(self.ui.sideWidget().stage, 1)

    def test_stage_progression_9(self):
        """Check that each page in the wizard sets the correct stage."""
        self.ui.on_currentIdChanged(self.ui.forgotten_password_page_id)
        self.assertEqual(self.ui.sideWidget().stage, 1)

    def test_stage_progression_10(self):
        """Check that each page in the wizard sets the correct stage."""
        self.ui.on_currentIdChanged(self.ui.reset_password_page_id)
        self.assertEqual(self.ui.sideWidget().stage, 1)

    def test_stage_progression_11(self):
        """Check that each page in the wizard sets the correct stage."""
        self.ui.on_currentIdChanged(self.ui.local_folders_page_id)
        self.assertEqual(self.ui.sideWidget().stage, 3)

    def test_stage_progression_12(self):
        """Check that each page in the wizard sets the correct stage."""
        self.ui.on_currentIdChanged(self.ui.SYNC_NOW_OR_LATER_PAGE)
        self.assertEqual(self.ui.sideWidget().stage, 3)

    def test_stage_progression_13(self):
        """Check that each page in the wizard sets the correct stage."""
        self.ui.on_currentIdChanged(self.ui.CONGRATULATIONS_PAGE)
        self.assertEqual(self.ui.sideWidget().stage, 4)

    def test_choose_signin_cancel(self):
        """Test that the custom cancel button works."""
        self.ui.sign_in_page.ui.cancel_button.click()
        QtCore.QCoreApplication.instance().processEvents()
        self.assertEqual(self.ui.result(), self.ui.Rejected)

    def test_license_next(self):
        """Test what's the license page's nextID."""
        self.ui.creds_cb("fakecreds")
        self.assertEqual(self.ui.LICENSE_PAGE.nextId(),
            self.ui.local_folders_page_id)

    def test_license_next_default(self):
        """By default, should be the sign in page."""
        self.assertEqual(self.ui.LICENSE_PAGE.nextId(),
            self.ui.SIGNIN_PAGE)


class FakeSignal(object):

    """A fake PyQt signal."""

    def __init__(self, *args, **kwargs):
        """Initialize."""
        self.target = lambda *args: None

    def connect(self, target):
        """Fake connect."""
        self.target = target

    def emit(self, *args):
        """Fake emit."""
        self.target(*args)


class FakeMainWindow(object):

    """A fake MainWindow."""

    loginSuccess = FakeSignal()
    registrationSuccess = FakeSignal()
    userCancellation = FakeSignal()
    shown = False

    def show(self):
        """Fake method."""
        self.shown = True


class FakeWizard(object):
    """Replace wizard() function on wizard pages."""

    params = None

    # Invalid name "setButtonLayout"
    # pylint: disable=C0103

    def setButtonLayout(self, *args, **kwargs):
        """Fake the functionality of setButtonLayout on QWizard class."""
        FakeWizard.params = (args, kwargs)

    # pylint: enable=C0103


class FakeWizard(object):
    """Replace wizard() function on wizard pages."""

    params = None

    # Invalid name "setButtonLayout"
    # pylint: disable=C0103

    def setButtonLayout(self, *args, **kwargs):
        """Fake the functionality of setButtonLayout on QWizard class."""
        FakeWizard.params = (args, kwargs)

    # pylint: enable=C0103


class SetupAccountTestCase(BaseTestCase):
    """Test the SetupAccountPage code."""

    class_ui = setup_account.SetupAccountPage
    kwargs = dict(
        ui=setup_account_ui.Ui_SetUpAccountPage(),
        controller=FakeController(),
        parent=None,
    )

    def setUp(self):
        """Initialize this test instance."""
        super(SetupAccountTestCase, self).setUp()

    def test_init(self):
        """Check the initial state of SetupAccount."""
        self.assertEqual(self.ui.ui.name_edit.receivers(
            QtCore.SIGNAL('textEdited(QString)')), 1)
        self.assertEqual(self.ui.ui.email_edit.receivers(
            QtCore.SIGNAL('textEdited(QString)')), 2)
        self.assertEqual(self.ui.ui.confirm_email_edit.receivers(
            QtCore.SIGNAL('textEdited(QString)')), 1)
        self.assertEqual(self.ui.ui.password_edit.receivers(
            QtCore.SIGNAL('textEdited(QString)')), 1)
        self.assertEqual(self.ui.ui.confirm_password_edit.receivers(
            QtCore.SIGNAL('textEdited(QString)')), 1)

    def test_initialize_page(self):
        """Check the initializePage to ensure proper widgets visibility."""
        self.ui.show()
        self.patch(self.ui, 'wizard', FakeWizard)
        self.ui.initializePage()
        self.assertEqual(FakeWizard.params, (([
            QtGui.QWizard.BackButton,
            QtGui.QWizard.Stretch],), {}))
        self.assertFalse(self.ui.ui.name_assistance.isVisible())
        self.assertFalse(self.ui.ui.email_assistance.isVisible())
        self.assertFalse(self.ui.ui.confirm_email_assistance.isVisible())
        self.assertFalse(self.ui.ui.password_assistance.isVisible())

    def test_empty_name(self):
        """Status when the name field is empty.

        * Name Assistance label visible.
        * Correct error message in it.
        """
        self.ui.ui.name_edit.setText("")
        self.ui.name_assistance()
        self.ui.show()
        self.assertEqual(self.ui.ui.name_assistance.isVisible(), True)
        self.assertEqual(
            unicode(self.ui.ui.name_assistance.text()),
            setup_account.ERROR % setup_account.EMPTY_NAME)
        self.ui.hide()

    def test_blank_name(self):
        """Status when the name field is blank (spaces).

        * Name Assistance label visible.
        * Correct error message in it.
        """
        self.ui.ui.name_edit.setText("  ")
        self.ui.name_assistance()
        self.ui.show()
        self.assertEqual(self.ui.ui.name_assistance.isVisible(), True)
        self.assertEqual(
            unicode(self.ui.ui.name_assistance.text()),
            setup_account.ERROR % setup_account.EMPTY_NAME)
        self.ui.hide()

    def test_valid_name(self):
        """Status when the name field is valid.

        * Name Assistance label invisible.
        """
        self.ui.ui.name_edit.setText("John Doe")
        self.ui.name_assistance()
        self.ui.show()
        self.assertEqual(self.ui.ui.name_assistance.isVisible(), False)
        self.ui.hide()

    def test_invalid_email(self):
        """Status when the email field has no @.

        * Email Assistance label visible.
        * Correct error message in it.
        """
        self.ui.ui.email_edit.setText("foobar")
        self.ui.email_assistance()
        self.ui.show()
        self.assertEqual(self.ui.ui.email_assistance.isVisible(), True)
        self.assertEqual(
            unicode(self.ui.ui.email_assistance.text()),
            setup_account.ERROR % setup_account.INVALID_EMAIL)
        self.ui.hide()

    def test_valid_email(self):
        """Status when the email field has a @.

        * Email Assistance label invisible.
        """
        self.ui.ui.email_edit.setText("foo@bar")
        self.ui.email_assistance()
        self.ui.show()
        self.assertEqual(self.ui.ui.email_assistance.isVisible(), False)
        self.ui.hide()

    def test_matching_emails(self):
        """Status when the email fields match.

        * Email Assistance label invisible.
        """
        self.ui.ui.email_edit.setText("foo@bar")
        self.ui.ui.confirm_email_edit.setText("foo@bar")
        self.ui.confirm_email_assistance()
        self.ui.show()
        self.assertEqual(
            self.ui.ui.confirm_email_assistance.isVisible(), False)
        self.ui.hide()

    def test_not_matching_emails(self):
        """Status when the email fields don't match.

        * Email Assistance label visible.
        * Correct error message.
        """
        self.ui.ui.email_edit.setText("foo@bar")
        self.ui.ui.confirm_email_edit.setText("foo@baz")
        self.ui.confirm_email_assistance()
        self.ui.show()
        self.assertEqual(self.ui.ui.confirm_email_assistance.isVisible(), True)
        self.assertEqual(
            unicode(self.ui.ui.confirm_email_assistance.text()),
            setup_account.ERROR % setup_account.EMAIL_MATCH)
        self.ui.hide()

    def test_short_password(self):
        """Status with short password.

        * Password assistance contains the right message.
        """
        self.ui.ui.password_edit.setText("foobar")
        self.ui.password_assistance()
        self.assertIn(
            setup_account.BAD % setup_account.PASSWORD_LENGTH,
            unicode(self.ui.ui.password_assistance.text()),
        )

    def test_long_password(self):
        """Status with long password.

        * Password assistance contains the right message.
        """
        self.ui.ui.password_edit.setText("foobarbaz")
        self.ui.password_assistance()
        self.assertIn(
            setup_account.GOOD % setup_account.PASSWORD_LENGTH,
            unicode(self.ui.ui.password_assistance.text()),
        )

    def test_no_number_password(self):
        """Status with password without a number.

        * Password assistance contains the right message.
        """
        self.ui.ui.password_edit.setText("foobarbaz")
        self.ui.password_assistance()
        self.assertIn(
            setup_account.BAD % setup_account.PASSWORD_DIGIT,
            unicode(self.ui.ui.password_assistance.text()),
        )

    def test_number_password(self):
        """Status with password with a number.

        * Password assistance contains the right message.
        """
        self.ui.ui.password_edit.setText("foobarba7")
        self.ui.password_assistance()
        self.assertIn(
            setup_account.GOOD % setup_account.PASSWORD_DIGIT,
            unicode(self.ui.ui.password_assistance.text()),
        )

    def test_no_uppercase_password(self):
        """Status with password without uppercase letters.

        * Password assistance contains the right message.
        """
        self.ui.ui.password_edit.setText("foobarbaz")
        self.ui.password_assistance()
        self.assertIn(
            setup_account.BAD % setup_account.PASSWORD_UPPER,
            unicode(self.ui.ui.password_assistance.text()),
        )

    def test_upper_password(self):
        """Status with password with uppercase letters.

        * Password assistance contains the right message.
        """
        self.ui.ui.password_edit.setText("Foobarba7")
        self.ui.password_assistance()
        self.assertIn(
            setup_account.GOOD % setup_account.PASSWORD_UPPER,
            unicode(self.ui.ui.password_assistance.text()),
        )

    def test_matching_passwords(self):
        """Status when the passwords match.

        * Password assistance doesn't contain the message.
        """
        self.ui.ui.password_edit.setText("Foobarba7")
        self.ui.ui.confirm_password_edit.setText("Foobarba7")
        self.ui.password_assistance()
        self.assertNotIn(
            setup_account.PASSWORD_MATCH,
            unicode(self.ui.ui.password_assistance.text()),
        )

    def test_not_matching_passwords(self):
        """Status when the passwords not match.

        * Password assistance contains the right message.
        """
        self.ui.ui.password_edit.setText("Foobarba7")
        self.ui.ui.confirm_password_edit.setText("BazBarFo0")
        self.ui.password_assistance()
        self.assertIn(
            setup_account.BAD % setup_account.PASSWORD_MATCH,
            unicode(self.ui.ui.password_assistance.text()),
        )

    def test_password_default_assistance(self):
        """Status when the password line edit receive focus and shows popup.

        * Password assistance contains the right message.
        """
        self.ui.ui.password_edit.setText("")
        self.ui.ui.confirm_password_edit.setText("")
        self.ui.password_assistance()
        self.ui.focus_changed(None, self.ui.ui.password_edit)
        self.assertIn(
            setup_account.NORMAL % setup_account.PASSWORD_LENGTH,
            unicode(self.ui.ui.password_assistance.text()),
        )
        self.assertIn(
            setup_account.NORMAL % setup_account.PASSWORD_UPPER,
            unicode(self.ui.ui.password_assistance.text()),
        )
        self.assertIn(
            setup_account.NORMAL % setup_account.PASSWORD_DIGIT,
            unicode(self.ui.ui.password_assistance.text()),
        )

    def test_password_assistance_in_focus_length_correct(self):
        """Check the QLineEdit for password when the length is correct."""
        self.ui.ui.password_edit.setText("aaaaaaaaa")
        self.ui.ui.confirm_password_edit.setText("")
        self.ui.password_assistance(setup_account.NORMAL)
        self.assertIn(
            setup_account.GOOD % setup_account.PASSWORD_LENGTH,
            unicode(self.ui.ui.password_assistance.text()),
        )
        self.assertIn(
            setup_account.NORMAL % setup_account.PASSWORD_UPPER,
            unicode(self.ui.ui.password_assistance.text()),
        )
        self.assertIn(
            setup_account.NORMAL % setup_account.PASSWORD_DIGIT,
            unicode(self.ui.ui.password_assistance.text()),
        )
        self.assertIn(
            setup_account.NORMAL % setup_account.PASSWORD_MATCH,
            unicode(self.ui.ui.password_assistance.text()),
        )

    def test_password_assistance_in_focus_with_upper(self):
        """Check the QLineEdit for password when it has an upper case char."""
        self.ui.ui.password_edit.setText("AAa")
        self.ui.ui.confirm_password_edit.setText("")
        self.ui.password_assistance(setup_account.NORMAL)
        self.assertIn(
            setup_account.NORMAL % setup_account.PASSWORD_LENGTH,
            unicode(self.ui.ui.password_assistance.text()),
        )
        self.assertIn(
            setup_account.GOOD % setup_account.PASSWORD_UPPER,
            unicode(self.ui.ui.password_assistance.text()),
        )
        self.assertIn(
            setup_account.NORMAL % setup_account.PASSWORD_DIGIT,
            unicode(self.ui.ui.password_assistance.text()),
        )
        self.assertIn(
            setup_account.NORMAL % setup_account.PASSWORD_MATCH,
            unicode(self.ui.ui.password_assistance.text()),
        )

    def test_password_assistance_in_focus_with_number(self):
        """Check the QLineEdit for password when it contains numbers."""
        self.ui.ui.password_edit.setText("a123")
        self.ui.ui.confirm_password_edit.setText("")
        self.ui.password_assistance(setup_account.NORMAL)
        self.assertIn(
            setup_account.NORMAL % setup_account.PASSWORD_LENGTH,
            unicode(self.ui.ui.password_assistance.text()),
        )
        self.assertIn(
            setup_account.NORMAL % setup_account.PASSWORD_UPPER,
            unicode(self.ui.ui.password_assistance.text()),
        )
        self.assertIn(
            setup_account.GOOD % setup_account.PASSWORD_DIGIT,
            unicode(self.ui.ui.password_assistance.text()),
        )
        self.assertIn(
            setup_account.NORMAL % setup_account.PASSWORD_MATCH,
            unicode(self.ui.ui.password_assistance.text()),
        )

    def test_password_assistance_in_focus_all_ok(self):
        """Check the QLineEdit for password when all the condition are ok."""
        self.ui.ui.password_edit.setText("T3st3rqw")
        self.ui.ui.confirm_password_edit.setText("T3st3rqw")
        self.ui.password_assistance(setup_account.NORMAL)
        self.assertIn(
            setup_account.GOOD % setup_account.PASSWORD_LENGTH,
            unicode(self.ui.ui.password_assistance.text()),
        )
        self.assertIn(
            setup_account.GOOD % setup_account.PASSWORD_UPPER,
            unicode(self.ui.ui.password_assistance.text()),
        )
        self.assertIn(
            setup_account.GOOD % setup_account.PASSWORD_DIGIT,
            unicode(self.ui.ui.password_assistance.text()),
        )
        self.assertNotIn(
            setup_account.NORMAL % setup_account.PASSWORD_MATCH,
            unicode(self.ui.ui.password_assistance.text()),
        )

    def test_side_widget_state(self):
        """Test if the side widget of the wizard has state frame visible."""
        if type(self.ui) != gui.MainWindow:
            return
        self.ui.on_currentIdChanged(self.ui.LICENSE_PAGE)
        self.assertEqual(
            True,
            self.ui.sideWidget().ui.states_frame.isVisible())
        self.ui.on_currentIdChanged(self.ui.SIGNIN_PAGE)
        self.assertEqual(
            True,
            self.ui.sideWidget().ui.states_frame.isVisible())
        self.ui.on_currentIdChanged(self.ui.local_folders_page_id)
        self.assertEqual(
            True,
            self.ui.sideWidget().ui.states_frame.isVisible())
        self.ui.on_currentIdChanged(self.ui.CONGRATULATIONS_PAGE)
        self.assertEqual(
            True,
            self.ui.sideWidget().ui.states_frame.isVisible())

    def test_check_valid(self):
        """Check the propery value of a QLineEdit marked as valid."""
        self.ui.check_as_valid(self.ui.ui.name_edit)
        self.assertEqual(self.ui.ui.name_edit.property("formError"),
            False)

    def test_check_invalid(self):
        """Check the propery value of a QLineEdit marked as invalid."""
        self.ui.check_as_invalid(self.ui.ui.name_edit)
        self.assertEqual(self.ui.ui.name_edit.property("formError"),
            True)

    def test_focus_changed(self):
        """Check visibility changes when focus_changed() is executed."""
        self.ui.show()
        self.ui.focus_changed(None, self.ui.ui.password_edit)
        self.assertTrue(self.ui.ui.password_assistance.isVisible())
        self.ui.focus_changed(None, self.ui.ui.captcha_solution_edit)
        self.assertTrue(self.ui.ui.refresh_label.isVisible())


class FakeFailingCredentialsManagementTool(object):

    """A fake CredentialsManagementTool that fails."""

    def find_credentials(self):
        """Return a deferred that never triggers callback."""
        return defer.fail(Exception())


class CredsFailureMainWindowTestCase(BaseTestCase):
    """Test the qt main window."""

    class_ui = gui.MainWindow

    def setUp(self):
        """Initialize this test instance."""
        # Faking each SSO object instead of doing it lower
        # so we don't rely on any SSO behaviour
        self.patch(gui, "SetUpAccountController", FakeController)
        self.patch(gui, "TosController", FakeController)
        self.patch(gui, "EmailVerificationController", FakeController)
        self.patch(gui, "CurrentUserController", FakeController)
        self.patch(gui, "SuccessController", FakeController)
        self.patch(gui, "ForgottenPasswordController", FakeController)
        self.patch(gui, "ResetPasswordController", FakeController)
        self.patch(gui, "AreYouSure", FakeAreYouSure)
        self.patch(gui, "LoadingOverlay", FakeOverlay)
        self.patch(gui, "LocalFoldersPage", FakeLocalFoldersPage)
        self.patch(qt.preferences, "PreferencesPanel", FakePreferencesPanel)
        self.patch(qt.folders, "FoldersPanel", FakeFoldersPanel)
        self.patch(gui, "CredentialsManagementTool",
            FakeFailingCredentialsManagementTool)
        super(CredsFailureMainWindowTestCase, self).setUp()

    def test_overlay_hiding(self):
        """Credential errors should make the overlay hide."""
        self.assertEqual(self.ui.overlay.hide_counter, 1)

    def test_critical(self):
        """Credential errors should display a critical message."""
        self.assertEqual(self.ui.form_errors_label.text(),
            "Application will close.\n\nException()")
